	package Nagios::CheckLogfiles::Search::Oraclealertlog;

use strict;
use Exporter;
use File::Basename;
use Time::Local;
use IO::File;
use vars qw(@ISA);

use constant OK => 0;
use constant WARNING => 1;
use constant CRITICAL => 2;
use constant UNKNOWN => 3;

@ISA = qw(Nagios::CheckLogfiles::Search);

sub new {
  my $self = bless {}, shift;
  return $self->init(shift);
}

sub init {
  my $self = shift;
  my $params = shift;
  $self->{logfile} = sprintf "%s/alertlog.%s", $self->{seekfilesdir},
      $self->{tag};
  $self->SUPER::init($params);
  $self->{oraalert}->{tns} = {
    connect => $params->{oraclealertlog}->{connect},
    username => $params->{oraclealertlog}->{username},
    password => $params->{oraclealertlog}->{password},
  };
}
    
sub prepare {
  my $self = shift;
  $self->{options}->{nologfilenocry} = 1;
  # the last second is the end time. in-progess seconds are not 
  # interesting yet.
  $self->{oraalert}->{untilsecond} = time - 1;
}

sub loadstate {
  my $self = shift;
  $self->SUPER::loadstate();
  # always scan the whole output. thst's what starttime is for.
  $self->{laststate}->{logoffset} = 0;
  # if this is the very first run, look back 5 mintes in the past.
  $self->{laststate}->{logtime} = $self->{laststate}->{logtime} ?
      $self->{laststate}->{logtime} : $self->{oraalert}->{untilsecond} - 60000;
}

sub savestate {
  my $self = shift;
  $self->{newstate} = $self->{laststate};
  # remember the last second scanned.
  $self->{newstate}->{logtime} = $self->{oraalert}->{untilsecond};
  $self->SUPER::savestate();
}

sub analyze_situation {
  my $self = shift;
  $self->trace("last scanned until %s", 
      scalar localtime $self->{laststate}->{logtime});
  if ($self->{laststate}->{logtime} < $self->{oraalert}->{untilsecond}) {
    $self->{logmodified} = 1;
  } else {
    $self->trace("please come back later");
  }
}

sub collectfiles {
  my $self = shift;
  my $fh = new IO::File;
  if ($self->{logmodified}) {
    # open database connection and select rows created since $self->{laststate}->{logtime}
    # and $self->{oraalert}->{untilsecond}
    my $linesread = 0;
    eval {
      require DBI;
      if (my $dbh = DBI->connect(
          sprintf("DBI:Oracle:%s", $self->{oraalert}->{tns}->{connect}),
          $self->{oraalert}->{tns}->{username},
          $self->{oraalert}->{tns}->{password}, 
          { RaiseError => 1, PrintError => 0 })) {
        my $sql = q{
            SELECT alert_timestamp, alert_text FROM alert_log 
            WHERE alert_timestamp > ? AND alert_timestamp <= ?
        };
        if (my $sth = $dbh->prepare($sql)) {
          $self->trace(sprintf "select events between %d and %d (%s and %s)",
              $self->{laststate}->{logtime}, $self->{oraalert}->{untilsecond},
              scalar localtime $self->{laststate}->{logtime},
              scalar localtime $self->{oraalert}->{untilsecond});
          $sth->execute($self->{laststate}->{logtime}, $self->{oraalert}->{untilsecond});
          if (my $fh = new IO::File($self->{logfile}, "w")) {
            while(my($alert_timestamp, $alert_text) = $sth->fetchrow_array()) {
              $fh->printf("%s %s\n", scalar localtime $alert_timestamp, $alert_text);
              $linesread++;
            }
            $fh->close();
          }
          $sth->finish();
        }
        $dbh->disconnect();
      }
    };
    if ($@) {
      $self->trace(sprintf "database operation failed: %s", $@);
      $self->addevent('UNKNOWN', sprintf "database operation failed: %s", $@);
    }
    $self->trace(sprintf "read %d lines from database", $linesread);
    if ($linesread) {
      if (my $fh = new IO::File($self->{logfile}, "r")) {
        $self->trace(sprintf "reopen logfile");
        push(@{$self->{relevantfiles}},
          { filename => "eventlog|",
            fh => $fh, seekable => 0,
            modtime => $self->{eventlog}->{nowminute},
            fingerprint => "0:0" });
      }
    }
  }
}

